<?php
/**
 * @package     CSVI
 * @subpackage  J2Store
 *
 * @author      RolandD Cyber Produksi <contact@csvimproved.com>
 * @copyright   Copyright (C) 2006 - [year] RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://csvimproved.com
 */

namespace j2store\com_j2store\model\import;

defined('_JEXEC') or die;

/**
 * Variants import.
 *
 * @package     CSVI
 * @subpackage  J2Store
 * @since       7.7.0
 */
class Variant extends \RantaiImportEngine
{
	/**
	 * CSVI fields
	 *
	 * @var    \CsviHelperImportFields
	 * @since  7.7.0
	 */
	protected $fields;

	/**
	 * The addon helper
	 *
	 * @var    \Com_J2StoreHelperCom_J2Store
	 * @since  7.7.0
	 */
	protected $helper;

	/**
	 * Variants table
	 *
	 * @var    \J2StoreTableVariant
	 * @since  7.7.0
	 */
	private $variantsTable;

	/**
	 * Start the menu import process.
	 *
	 * @return  bool  True on success | false on failure.
	 *
	 * @since   7.7.0
	 *
	 * @throws  \RuntimeException
	 * @throws  \InvalidArgumentException
	 * @throws  \UnexpectedValueException
	 */
	public function getStart()
	{
		// Process data
		foreach ($this->fields->getData() as $fields)
		{
			foreach ($fields as $name => $details)
			{
				$value = $details->value;

				switch ($name)
				{
					case 'parent_product_id':
						$this->setState('product_id', $value);
						break;
					case 'shipping':
					case 'is_master':
					case 'use_store_config_min_out_qty':
					case 'use_store_config_min_sale_qty':
					case 'use_store_config_max_sale_qty':
					case 'use_store_config_notify_qty':
					case 'allow_backorder':
					case 'isdefault_variant':
						switch (strtolower($value))
						{
							case 'n':
							case 'no':
							case '0':
								$value = 0;
								break;
							default:
								$value = 1;
								break;
						}

						$this->setState($name, $value);
						break;
					default:
						$this->setState($name, $value);
						break;
				}
			}
		}

		$sku = $this->getState('sku', false);

		if (!$sku)
		{
			$this->loaded = false;
			$this->log->addStats('skipped', 'COM_CSVI_NO_PRODUCT_SKU_FOUND');
		}
		else
		{
			$this->loaded = true;
			$variantId    = $this->helper->getVariantId($sku);

			if ($variantId)
			{
				$this->setState('j2store_variant_id', $variantId);
			}

			if ($variantId && $this->variantsTable->load($this->getState('j2store_variant_id', 0)))
			{
				if (!$this->template->get('overwrite_existing_data'))
				{
					$this->log->add(\JText::sprintf('COM_FIELDS_WARNING_OVERWRITING_SET_TO_NO', $this->getState('sku')), false);
					$this->loaded = false;
				}
			}
		}

		return true;
	}

	/**
	 * Process a record.
	 *
	 * @return  bool  Returns true if all is OK | Returns false if no path or menu ID can be found.
	 *
	 * @since   7.7.0
	 *
	 * @throws  \RuntimeException
	 * @throws  \InvalidArgumentException
	 * @throws  \UnexpectedValueException
	 */
	public function getProcessRecord()
	{
		if (!$this->loaded)
		{
			return false;
		}

		if (!$this->getState('j2store_variant_id', false) && $this->template->get('ignore_non_exist'))
		{
			$this->log->addStats('skipped', \JText::sprintf('COM_CSVI_DATA_EXISTS_IGNORE_NEW', $this->getState('sku', '')));
		}
		else
		{
			if (!$this->getState('pricing_calculator', false))
			{
				$this->setState('pricing_calculator', 'standard');
			}

			if (!$this->getState('is_master', false))
			{
				$this->setState('is_master', 0);
			}
			
			$this->variantsTable->bind($this->state);
			$this->variantsTable->check();

			try
			{
				$this->variantsTable->store();
				$this->log->add('Variants added successfully', false);
			}
			catch (\Exception $e)
			{
				$this->log->add('Cannot add Variants. Error: ' . $e->getMessage(), false);
				$this->log->addStats('incorrect', $e->getMessage());

				return false;
			}

		}

		return true;
	}

	/**
	 * Load the necessary tables.
	 *
	 * @return  void.
	 *
	 * @since   7.7.0
	 */
	public function loadTables()
	{
		$this->variantsTable = $this->getTable('Variant');
	}

	/**
	 * Clear the loaded tables.
	 *
	 * @return  void.
	 *
	 * @since   7.7.0
	 */
	public function clearTables()
	{
		$this->variantsTable->reset();
	}
}
